#ifndef STACK_HPP_INCLUDED
#define STACK_HPP_INCLUDED

template <class T> // elemtípus sablonja
class Stack // verem típus
{
    private:
        T* _values; // értékek mutatója
        int _top; // a tetõelem indexe
        int _size; // méret eltárolása
    public:
        enum Exceptions { BAD_SIZE, STACK_FULL, STACK_EMPTY }; // kivételek felsorolási típusa

        Stack(int size); // konstruktor
        ~Stack(); // destruktor
        bool isEmpty() const; // üres-e a verem
        bool isFull() const; // tele van-e a verem
        void push(T value); // elem behelyezése
        T pop(); // elem kivétele
        T top() const; // tetõelem lekérdezése
        void clear(); // elemek törlése
        int size() const; // méret lekérdezése
};

template <class T>
Stack<T>::Stack(int size)
{
    if (size <= 0) // ellenőrizzük a paramétert
        throw BAD_SIZE;

    _values = new T[size]; // tömb dinamikus létrehozása
    _top = 0; // kezdetben 0 elem van a veremben
    _size = size;
}

template <class T>
Stack<T>::~Stack()
{
    delete[] _values; // dinamikus tömb törlése
}

template <class T>
bool Stack<T>::isEmpty() const
{
    return _top == 0;
}

template <class T>
bool Stack<T>::isFull() const
{
    return _top == _size;
}

template <class T>
void Stack<T>::push(T value)
{
    if (_top < _size) // ha még van hely
    {
        _values[_top] = value; // behelyezzük az elemet
        _top++; // növeljük az elemszámot
    }
    else // különben kivételt dobunk
        throw STACK_FULL;
}

template <class T>
T Stack<T>::pop()
{
    if (_top > 0) // ha van elem
    {
        _top--; // csökkentjük az elemszámot
        return _values[_top]; // visszaadjuk a kivett elemet
    }
    else // különben kivételt dobunk
        throw STACK_EMPTY;
}

template <class T>
T Stack<T>::top() const
{
    if (_top > 0) // ha van elem
        return _values[_top - 1];
    else // különben kivételt dobunk
        throw STACK_EMPTY;
}

template <class T>
void Stack<T>::clear()
{
    _top = 0;
}

template <class T>
int Stack<T>::size() const
{
    return _top;
}

#endif // STACK_HPP_INCLUDED
